#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import open_project, save_project, files, sequences
import thumbnail as thumbnail
import os, sys
import xml.dom.minidom as xml
import profiles

########################################################################
class project():
	"""This is the main project class that contains all
	the details of a project, such as name, folder, timeline
	information, sequences, media files, etc..."""

	#----------------------------------------------------------------------
	def __init__(self):
		"""Constructor"""
		# get the path of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0]))
		
		# init the variables for the project
		self.name = "Default Project"
		self.folder = path
		self.project_type = "DV NTSC"
		self.theme = "blue_glass1"
		self.canvas = None
		
		# reference to the main GTK form
		self.form = None
		
		# init the file / folder list (used to populate the tree)
		self.project_folder = files.OpenShotFolder(self)

		# ini the sequences collection
		self.sequences = [sequences.sequence("Default Sequence 1", self)]		

		# init the tab collection
		self.tabs = [self.sequences[0]]	  # holds a refernce to the sequences, and the order of the tabs

		# clear temp folder
		self.clear_temp_folder()
		
		# create thumbnailer object
		self.thumbnailer = thumbnail.thumbnailer()
		self.thumbnailer.set_project(self)
		self.thumbnailer.start()
		
	def fps(self):
		# get the profile object
		fps = profiles.mlt_profiles().get_profile(self.project_type).fps()

		# return the frames per second
		return fps

	def clear_temp_folder(self):
		"""This method deletes all files in the /openshot/temp folder."""
		path = os.path.dirname(os.path.abspath(sys.argv[0]))
		
		# create thumbnail folder (if it doesn't exist)
		if os.path.exists("%s/thumbnail" % path) == False:
			# create new thumbnail folder
			os.mkdir("%s/thumbnail" % path)
		
		# loop through all files in temp folder
		for file_name in os.listdir("%s/thumbnail" % path):
			if file_name.endswith(".txt") == False:
				# remove the tmp thumbnail files
				os.remove("%s/thumbnail/%s" % (path, file_name))

	#----------------------------------------------------------------------
	def Render(self):
		"""This method recursively renders all the tracks and clips on the timeline"""
		
		# Render the timeline
		self.sequences[0].Render()
		
		# Render Play Head (and position line)
		self.sequences[0].RenderPlayHead()
		
		
	def GenerateXML(self, file_name):
		"""This method creates the MLT Westley XML used by OpenShot"""
		path = os.path.dirname(os.path.abspath(sys.argv[0]))
		
		# Create the XML document
		dom = xml.Document()

		# Add the root element
		westley_root = dom.createElement("westley")
		dom.appendChild(westley_root)
		tractor1 = dom.createElement("tractor")
		tractor1.setAttribute("id", "tractor0")
		westley_root.appendChild(tractor1)
		
		# Add all the other timeline objects (such as sequences, clips, filters, and transitions)
		self.sequences[0].GenerateXML(dom, tractor1)
		
		# Save the XML dom
		f = open("%s/%s" % (path, file_name), "w")
		dom.writexml(f)
		f.close()

	
	#----------------------------------------------------------------------
	def Save(self, file_path):
		"""Call the save method of this project, which will 
		persist the project to the file system."""
		
		# call the save method
		save_project.save_project(self, file_path)
		
	#----------------------------------------------------------------------
	def Open(self, file_path):
		"""Call the open method, which will open an existing
		project file from the file system."""
			
		# call the open method
		open_project.open_project(self, file_path)	
		
		