#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import sys, os
import shutil
import gtk
import cPickle as pickle



def save_project(project_object, file_path):
	
		# determine location of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0]))

		# create thumbnail folder (if it doesn't exist)
		if os.path.exists("%s/thumbnail" % project_object.folder) == False:
			# create new thumbnail folder
			os.mkdir("%s/thumbnail" % project_object.folder)
		
		# copy all temp thumbnails to this project's folder
		for file_name in os.listdir("%s/thumbnail" % path):
			if file_name.endswith(".txt") == False:
				# determine target copy location
				target_path = "%s/thumbnail/%s" % (project_object.folder, file_name)
				
				# copy the thumbnail to the new location (if it doesn't exit)
				if os.path.exists(target_path) == False:
					shutil.copyfile("%s/thumbnail/%s" % (path, file_name), target_path)
		
		# Loop through the files, and update the thumbnail location
		for item in project_object.project_folder.items:

			file_name = item.name
			thumb_location = "%s/thumbnail/%s" % (project_object.folder, file_name)
			
			(dirName, file_name) = os.path.split(thumb_location)
			(fileBaseName, fileExtension) = os.path.splitext(file_name) 
			
			new_thumbnail_location = os.path.join(dirName, fileBaseName + ".jpg")
			item.thumb_location = new_thumbnail_location

		# clear the following temporary properties which can't be pickeled
		old_form = project_object.form
		old_play_head = project_object.sequences[0].play_head
		old_ruler_time = project_object.sequences[0].ruler_time
		old_thumbnailer = project_object.thumbnailer
		
		project_object.sequences[0].play_head = None
		project_object.sequences[0].ruler_time = None
		project_object.form = None
		project_object.thumbnailer = None

		# serialize the project object
		myFile = file(file_path, "wb")
		pickle.dump(project_object, myFile, True)
		
		# re-attach some variables (that aren't pickleable)
		project_object.form = old_form
		project_object.sequences[0].play_head = old_play_head
		project_object.sequences[0].ruler_time = old_ruler_time
		project_object.thumbnailer = old_thumbnailer
		
		# update the thumbnailer's project reference
		project_object.thumbnailer.set_project(project_object)
		
		# update project reference to menus
		project_object.form.mnuTrack1.project = project_object
		project_object.form.mnuClip1.project = project_object
		
		manager = gtk.recent_manager_get_default()
		manager.add_item('file://' + file_path)
		
		# output the file path
		print "project saved! - %s" % (project_object.name)
			   
		
	#except IOError:
	#	# print error message
	#	print "*** error saving file ***"
		  
		  