#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import mltpp
import os, sys
import threading
import time
import files as files


class thumbnailer ( threading.Thread ):
	""" This class is designed to always be running during OpenShot.  It's a seperate thread that 
	is always waiting to inspect video and audio files, generate thumbnails, etc... """

	def set_project(self, project):
		 """ Associate the OpenShot project file with this threaded class. """
		 self.project = project
		 
	def GetFile(self, file_name):
		 """ Use this method to generate an OpenShotFile object based on the URL (or file location)
		 of a video or audio file. Each time you call this method, it will lock this thread (and OpenShot's
		 main thread) until it has finished. """

		 try:
			 	
			 # determine name and location of thumbnail image
			 self.file_name = file_name
			 self.thumbnail_path = ""
			 project_path = self.project.folder
			 myPath = self.file_name
			 (dirName, fileName) = os.path.split(myPath)
			 (fileBaseName, fileExtension)=os.path.splitext(fileName)
			 self.thumbnail_path = project_path + "/thumbnail/" + fileBaseName + ".jpg"

			 # Create the producer
			 self.p = mltpp.Producer( self.profile, '%s' % self.file_name )
			 self.c = mltpp.Consumer(self.profile, "avformat", self.thumbnail_path)
			 self.c.set("real_time", 0)
			 self.c.set("progressive", 1)

			 # determine length of clip in seconds
			 max_frames = float(self.p.get_length())
			 producer_fps = float(self.p.get_fps())
			 calculate_length = max_frames / producer_fps

			 # determine if this is an image
			 is_image = False
			 if self.p.get_length() == 15000:
			 	# images always have exactly 15000 frames
			 	is_image = True
			 	
			 	# set the length of the image to 10 seconds
			 	calculate_length = float(30)

			 height = 0
			 width = 0
			 if self.p.get("height"):
				  height = int(self.p.get("height"))
			 if self.p.get("width"):
				  width = int(self.p.get("width"))
	
			 # set thumbnail image (if no height & width are detected)
			 if (height == False or width == False) and (is_image == False):
				  self.thumbnail_path = ""
			 
			 # get the 1st frame
			 self.p = self.p.cut(1, 1)
			 
			 # connect the producer and consumer
			 self.c.connect( self.p )

			 # Start the consumer, and lock the thread until it's done (to prevent crazy seg fault errors)
			 # Only start if the media item has a thumbnail location (i.e. no audio thumbnails)
			 if self.thumbnail_path:
			 	self.c.run()
	
			 # create an openshot file object
			 newFile = files.OpenShotFile(self.project)
			 newFile.name = fileName
			 newFile.location = self.file_name
			 newFile.length = calculate_length
			 newFile.length_in_seconds = newFile.length
			 newFile.thumb_location = self.thumbnail_path
			 newFile.videorate = (self.p.get_fps(), 0)
			 newFile.height = height
			 newFile.width = width
			 newFile.max_frames = max_frames
			 newFile.fps = producer_fps

			 # return the OpenShotFile object
			 return newFile
		
		 except Exception:
			print "Failed to import file: %s" % file_name



	def run ( self ):
		 """ This is the main method on this thread.  This method should not return anything, or the 
		 thread will no longer be active... and thus will no longer be able to inspect media files. """
		 
		 self.amAlive = True
		 self.file_name = ""
		 self.c = None
		 self.p = None

		 # init the factory, and load a small video size / profile
		 mltpp.Factory().init()
		 self.profile = mltpp.Profile("quarter_ntsc")

		 # this loop will continue as long as OpenShot is running
		 while self.amAlive:
			  time.sleep( 1 ) 
