#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import threading
import os, sys
import profiles

class player ( threading.Thread ):
	
	def __init__(self, project, main_form, file_name, mode=None):
		self.project = project
		self.main_form = main_form
		self.file_name = file_name
		
		# set the FPS
		self.fps = self.project.fps()
		
		# update mode
		if mode:
			# set the mode
			self.mode = mode
			
		# call base class
		threading.Thread.__init__(self)

	def set_project(self, project, main_form, file_name, mode=None):
		self.project = project
		self.main_form = main_form
		self.file_name = file_name
		
		# set the FPS
		self.fps = self.project.fps()
		
		# update mode
		if mode:
			# set the mode
			self.mode = mode
		
	def set_profile(self, profile_name):

		# set the MLT profile object... specifiec in the project properties
		self.profile = profiles.mlt_profiles().get_profile(profile_name)

		# set the FPS
		self.fps = self.project.fps()
		
		# re-load the mlt objects (i.e. the producer & consumer)
		self.load_xml()
		
		
	def load_xml(self):
		
		import sys
		# determine location of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0]))		
		
		# Import required modules
		import mltpp
		import time
		import sys
		import gobject
		
		# set the FPS
		self.fps = self.project.fps()
		
		# stop the consumer
		self.c.stop()
		
		# Create the producer
		self.p = mltpp.Producer( self.profile, 'westley:%s/%s' % (path, self.file_name))

		# Create the consumer
		if self.mode == "render":
			# RENDER MOVIE mode
			self.c = mltpp.Consumer( self.profile, "avformat", "%s/%s.mpeg" % (self.project.folder, self.project.name))
			
			# set some RENDER specific options
			self.c.set("real_time", 0)
			
			self.c.set( "b", "5000000" )			# bit rate: default is 200,000 (i.e. 200 kb/sec)
			#self.c.set( "f", "avi")				# container format
			#self.c.set( "vcodec", "rawvideo")	# video codec
			#self.c.set ( "vcodec", "vorbis" ) 			
			#self.c.set( "acodec", "pcm_s16le")	# audio codec
			#self.c.set( "acodec", "mp3")
			#self.c.set( "ar", "44100" )			# audio / sample rate
			self.c.set( "ac", "2" )					# audio channels
			#self.c.set( "r", "25" )				# frames per second
	
		else:
			# PREVIEW mode
			self.c = mltpp.Consumer( self.profile, "sdl" )
			self.c.set("real_time", 1)
		
		# set some consumer options
		self.c.set("rescale", "1" )
		self.c.set("distort", "1" )
		self.c.set("progressive", 1)

		# connect the producer and consumer
		self.c.connect( self.p )
		
		# if render mode, always play the consumer
		if self.mode == "render" and self.c.is_stopped():
			
			# start consumer
			self.c.start()
			

	def run ( self ):
		import sys
		# determine location of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0])) 

		# Import required modules
		import mltpp
		import time
		import sys
		import gobject

		# track wheather the thread is playing or not
		self.isPlaying = True 
		
		# track if this thread should die
		self.amAlive = True
		
		# Start the mlt system
		self.f = mltpp.Factory().init( )
		
		# set the MLT profile object... specifiec in the project properties
		self.profile = profiles.mlt_profiles().get_profile(self.project.project_type)
		
		# Create the producer
		self.p = mltpp.Producer( self.profile, 'westley:%s/%s' % (path, self.file_name))
		
		if self.p:
			
			# Create the consumer
			if self.mode == "render":
				# RENDER MOVIE mode
				self.c = mltpp.Consumer( self.profile, "avformat", "%s/%s.mpeg" % (self.project.folder, self.project.name))
			else:
				# PREVIEW mode
				self.c = mltpp.Consumer( self.profile, "sdl" )
		
			# set some consumer options
			self.c.set("rescale", "1" )
			self.c.set("distort", "1" )
			self.c.set("real_time", 1)
			self.c.set("progressive", 1)
			
			# Connect the producer to the consumer
			self.c.connect( self.p )
			
			# Start the consumer
			self.c.start( )
			
			# Get the FPS
			self.fps = self.project.fps()
			
			
			# Wait until the user stops the consumer
			while self.amAlive:
				# only calculate position / percentage when playing
				if self.c.is_stopped() == False:

					# move play head
					new_time = float(self.p.position()) / float(self.fps)
					gobject.idle_add(self.project.sequences[0].move_play_head, new_time)
	
					# update progress bar of video 
					percentage_complete = (float(self.p.position()) / float(self.p.get_length() - 1)) * float(100)
					gobject.idle_add(self.main_form.hsVideoProgress.set_value, percentage_complete)
					
					# wait 1/5 of a second
					time.sleep( .2 )
					
				else:

					# wait 1 second
					time.sleep( 1 )


			# clear all the MLT objects
			self.c.stop()
			self.p = None
			self.c = None
			self.profile = None
			self.f = None
			
		else:
			# Diagnostics
			print "ERROR WITH %s" % self.file_name


	def pause(self):
		# pause the video
		self.isPlaying = False
		self.p.set_speed(0)
			
	def play(self):
		# play the video
		self.isPlaying = True
		self.p.set_speed(1)
		
		# check if consumer is started
		if self.c.is_stopped():
			self.c.start()

		

	def seek(self, frame_number):
		self.p.seek(frame_number) 
