#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.



#   ----------------------------- EXAMPLE CODE ----------------------------
#   Use this snippet on any module that needs to support translations.  Once this 
#   snippet is in place, you can use the _("") method to set all your strings.
#
#       import language.Language_Init as Language_Init
#
#       # Add this to each method that needs to translate strings
#       _ = Language_Init.Translator().lang.gettext
#
#       # To mark a string as 'translatable', use this syntax:
#       MyString = _("Here is my string")



# ----------------------- Translation Example Commands --------------------
# To translate this program, you must create a POT (template) file, then a
# PO file (Ascii), then a MO file (binary), and then add it to the correct
# locale folder in the /language/{CODE}/LC_MESSAGES/ folder.
#
# COMMANDS
# ------------------------------------------------------------------------
# Command to create the POT Template file, recursively for all .py files in the /openshot/ project.
# NOTE:  The OpenShot.pot file must already exist, or you will get errors.
#        $ find ~/openshot/ -iname "*.py" -exec xgettext -C -j -o ~/openshot/main/language/OpenShot.pot --keyword=_ {} \;

# Command to create the individual language PO files (Ascii files)
#        $ msginit --input=OpenShot.pot --locale=en_US
#        $ msginit --input=OpenShot.pot --locale=es

# Command to update the PO files (if text is added or changed)
#        $ msgmerge en_US.po OpenShot.pot -U
#        $ msgmerge es.po OpenShot.pot -U

# Command to compile the Ascii PO files into binary MO files
#        $ msgfmt en_US.po --output-file=en_US/LC_MESSAGES/OpenShot.mo
#        $ msgfmt es.po --output-file=es/LC_MESSAGES/OpenShot.mo

# OR, install POedit (http://www.poedit.net/), and open the .PO files with this program.  
# It let's you type the translations, saves the .PO file, and it automatically creates the .MO files
# for you.  Very nice.

# Then rename and copy the MO files into the /languages/{en_us/es/fr/etc}/LC_MESSAGES/OpenShot.mo
# NOTE:  BE SURE TO RENAME THE FILE = OpenShot.mo

# Command to Launch Program in any language
# LANG=es python OpenShot.py
# -------------------------------------------------------------------------


# import modules needed for language translation & gtk support
import gettext
import locale
import os, sys

########################################################################
class Translator:
    """Class that contains functions to initiate language translations"""  
    
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""  
              
        # Setup foriegn lanugage support
        gettext.install(domain="OpenShot", unicode = False, codeset = 'utf-8')
        local_path = os.path.realpath(os.path.dirname(sys.argv[0]))
        langs = []
        lc, encoding = locale.getdefaultlocale()
        if (lc):
            langs = [lc]
            
        language = os.environ.get('LANGUAGE', None)
        if (language):
            langs += language.split(":")
        # Add languages... for testing
        langs += ["en_US", "es"]
        
        gettext.bindtextdomain("OpenShot", local_path)
        gettext.textdomain("OpenShot")
        
        # This reference is used by other classes to define the _ method 
        self.lang = gettext.translation("OpenShot", local_path + "/language/", languages = langs, fallback = True)        


