#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

from classes.project import project
from classes.messagebox import *

import gtk, gtk.glade
import os
from SimpleGladeApp import SimpleGladeApp
from windows.MainGTK import *
import pango

# init the foriegn language
import language.Language_Init as Language_Init


class frmFileproperties(SimpleGladeApp):

	def __init__(self, file, path="FileProperties.glade", root="frmFileProperties", domain="OpenShot", form=None, project=None, **kwargs):
		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator().lang.gettext
		
		self.form = form
		self.project = project
		
		#generic properties
		self.lblName.set_label(file.name)
		if file.thumb_location != "":
			pixbuf = gtk.gdk.pixbuf_new_from_file(file.thumb_location)
			pixbuf = pixbuf.scale_simple(102,76,gtk.gdk.INTERP_BILINEAR)
			self.imgPreview.set_from_pixbuf(pixbuf)
		self.lblMimeType.set_label(file.mimetype)
		#some mimetypes are quite long (e.g. mpeg), they'll never fit in the label space.
		#add a tooltip to contain the mimetype, and set ellipses if the lable doesn't fit.
		self.lblMimeType.set_tooltip_text(file.mimetype)
		self.lblMimeType.set_ellipsize(pango.ELLIPSIZE_END)
		self.lblLocation.set_label(file.location)
		self.lblLengthValue.set_label(str(file.length_in_seconds) + " secs")
		self.txtLabel.set_text(file.label)
		
		if file.mimetype.startswith("video"):
		#video specific properties
			self.lblRateValue.set_label(str(file.videorate[0]) + "/" + str(file.videorate[1]))
			self.lblSizeValue.set_label(str(file.width) + " x " + str(file.height))
			
		elif file.mimetype.startswith("audio"):
		#audio specific properties
			self.lblRate.set_label("Audio Depth")
			self.lblRateValue.set_label(str(file.audiodepth))
			self.lblSize.set_label("Audio width")
			self.lblSizeValue.set_label(str(file.audiowidth))
		
		self.lblSampleRateValue.set_label(str(file.audiorate))
		self.lblAudioChannelsValue.set_label(str(file.audiochannels))
		
		self.frmFileProperties.show_all()
		
	def on_btnClose_clicked(self, event):
		self.frmFileProperties.destroy()
		
	def on_btnApply_clicked(self, event):
		if self.txtLabel.get_text() == "":
			return
		self.project.project_folder.UpdateFileLabel(self.lblName.get_label(), self.txtLabel.get_text(), 1)
		self.frmFileProperties.destroy()
		
def main():
	frm_file_properties = frmFileProperties()
	frm_file_properties.run()

if __name__ == "__main__":
	main()
