#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.


import gtk, gtk.glade
import os, sys
import operator
import goocanvas
from SimpleGladeApp import SimpleGladeApp

import classes.project
import classes.video as video
import NewProject, OpenProject, About, FileProperties
import classes.tree as tree
import classes.timeline as timeline
import classes.files as files
import classes.messagebox as messagebox

import AddFiles
import pango

# init the foriegn language
import language.Language_Init as Language_Init


class frmMain(SimpleGladeApp):

	def __init__(self, path="Main.glade", root="frmMain", domain="OpenShot", project=None, **kwargs):
		"""Init the main window"""

		self.MyVideo = None

		# Load the Glade form using the SimpleGladeApp module	
		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator().lang.gettext
		
		# project instance
		self.project = project
		self.project.form = self
		
		# determine the directory OpenShot is running in.  This is used 
		# to correctly load images, themes, etc...
		self.openshot_path = os.path.dirname(os.path.abspath(sys.argv[0]))

		# variable for timeline drag n drop
		self.dragging = False
		self.timeline_clip_y = 0

		# Init track variables
		self.AllTracks = []
		(self.CurrentTrack_X, self.CurrentTrack_Y) = 10, 10
		(self.Scroll_Vertical_Value, self.Scroll_Horizontal_Value) = 10, 10

		# Create the Canvas Widget
		hboxTimeline = self.scrolledwindow_Left
		isinstance(hboxTimeline, gtk.HBox)

		# Get reference to the window
		self.App1Window = self.frmMain
		isinstance(self.App1Window, gtk.Window)

		# Create new GooCanvas (and add to form)
		self.MyCanvas_Left = goocanvas.Canvas()
		self.MyCanvas_Left.connect("scroll-event", self.on_scrolledwindow_Left_scroll_event)
		self.MyCanvas_Left.set_size_request (160, 0)
		self.MyCanvas_Left.set_bounds (0, 0, 160, 1000)
		self.MyCanvas_Left.show()

		self.MyCanvas = goocanvas.Canvas()
		self.MyCanvas.connect("scroll-event", self.on_scrolledwindow_Right_scroll_event)
		self.MyCanvas.set_bounds (0, 0, 50000, 1000)
		hboxTimeline.set_border_width(0)
		self.MyCanvas.show()

		self.scrolled_win = self.scrolledwindow_Left
		self.scrolled_win.add(self.MyCanvas_Left)
		self.scrolled_win.show()	  

		self.scrolled_win_Right = self.scrolledwindow_Right
		self.scrolled_win_Right.add(self.MyCanvas)
		self.scrolled_win_Right.show()

		# Add default node to the tree
		self.myTree = self.treeFiles
		treestore = gtk.TreeStore
		NoItemSelected = gtk.TreeIter

		# create a TreeStore
		self.MyTreestore = gtk.TreeStore(gtk.gdk.Pixbuf, str, str, str)
		
		#set multiple selection mode on the tree
		selection = self.myTree.get_selection()
		selection.set_mode(gtk.SELECTION_MULTIPLE)
		
		#set multiple selection on the iconview
		self.icvFileIcons.set_selection_mode(gtk.SELECTION_MULTIPLE)

		# Set the treeview's data model
		self.myTree.set_model(self.MyTreestore)
		tree.treeviewAddGeneralPixbufColumn(self.myTree, "Thumb", 0, resizable=False, reorderable=False, project=self.project)
		tree.treeviewAddGeneralTextColumn(self.myTree, "File", 1, resizable=False, reorderable=True, editable=False, visible=True, elipses=False, autosize=True, project=self.project)
		tree.treeviewAddGeneralTextColumn(self.myTree, "Length", 2, resizable=True, reorderable=True, editable=False, visible=True, project=self.project)
		tree.treeviewAddGeneralTextColumn(self.myTree, "Label", 3, resizable=True, reorderable=True, editable=True, visible=True, elipses=True, project=self.project)
		
		
		model = self.myTree.get_model()
		item = model.append(None)
		model.set_value(item, 0, None)
		model.set_value(item, 1, _("Choose a Video or Audio File to Begin"))
		model.set_value(item, 2, "")
		model.set_value(item, 3, "")

		
	   
		#Add a recent projects menu item
		manager = gtk.recent_manager_get_default()

		recent_menu_chooser = gtk.RecentChooserMenu(manager)
		recent_menu_chooser.set_limit(5)
		recent_menu_chooser.set_sort_type(gtk.RECENT_SORT_MRU)
		filter = gtk.RecentFilter()
		filter.add_pattern("*.osp")
		recent_menu_chooser.add_filter(filter)
		recent_menu_chooser.connect('item-activated', self.recent_item_activated)

		mnurecent = self.mnuRecent
		mnurecent.set_submenu(recent_menu_chooser)

		###################
		
		# init the track menu
		self.mnuTrack1 = mnuTrack(None, None, form=self, project=self.project)
		
		# init the clip menu
		self.mnuClip1 = mnuClip(None, None, form=self, project=self.project)
		
		# init the marker menu
		self.mnuMarker1 = mnuMarker(None, None, form=self, project=self.project)
		
		###################

		self.TARGET_TYPE_URI_LIST = 80
		dnd_list = [ ( 'text/uri-list', 0, self.TARGET_TYPE_URI_LIST ) ]	

		# Enable drag n drop on the Treeview and Canvas
		isinstance(self.myTree, gtk.TreeView)
		self.myTree.connect_after('drag_begin', self.on_treeFiles_drag_begin)
		self.myTree.connect('drag_data_received', self.on_drag_data_received)
		self.myTree.drag_dest_set( gtk.DEST_DEFAULT_MOTION |
										 gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
								   dnd_list, gtk.gdk.ACTION_COPY)

		self.icvFileIcons.connect('drag_data_received', self.on_drag_data_received)
		self.icvFileIcons.drag_dest_set( gtk.DEST_DEFAULT_MOTION |
										 gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
										 dnd_list, gtk.gdk.ACTION_COPY)
		
		self.icvFileIcons.connect_after('drag_begin', self.on_treeFiles_drag_begin)
		
		self.icvFileIcons.enable_model_drag_source(gtk.DEST_DEFAULT_MOTION |
										 gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,
										 dnd_list, gtk.gdk.ACTION_COPY)


		self.MyCanvas.drag_dest_set(0, [], 0)
		self.MyCanvas.connect('drag_motion', self.motion_cb)
		self.MyCanvas.connect('drag_drop', self.drop_cb)
		self.MyCanvas.connect('motion_notify_event', self.canvas_motion_notify)
		self.last_files_added = ""
		
		# track the cursor, and what position it was last changed
		self.current_cursor = [None, 0, 0, ""]  # Cursor Name, X, Y, Cursor Source

		
		# track when a dragged item reaches the goocanvas
		self.item_detected = False
		self.new_clip = None
		self.new_clip_object = None



	def recent_item_activated(self, widget):
		"""Activated when an item from the recent projects menu is clicked"""
		import urllib
		
		uri = widget.get_current_item().get_uri()
		
		# clean path to video
		#uri = uri.replace("%20", " ")
		uri = urllib.unquote(uri)
		
		# Strip 'file://' from the beginning
		file_to_open = uri[7:]
		self.project.Open(file_to_open)

		# Update the main form
		self.refresh() 


	def new(self):
		print "A new %s has been created" % self.__class__.__name__


	def refresh(self):
		
		# determine location of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0]))
		
		"""Called when the Treeview is active"""
		# Set the title of the window
		self.frmMain.set_title("OpenShot - %s" % (self.project.name))
		
		# Set the zoom scale
		self.hsZoom.set_value(self.project.sequences[0].scale)

		# Clear the file treeview
		self.MyTreestore.clear()
		
		# render timeline
		self.project.Render()
			
		#sort the list of items so parent folders are added before
		#the child items, otherwise files that belong to a folder won't get added.
		items = self.project.project_folder.items
		items.sort(key=operator.attrgetter('parent'))

		# Loop through the files, and add them to the project tree
		for item in items:
			if isinstance(item, files.OpenShotFile):
				#format the file length field
				milliseconds = item.length_in_seconds * 1000
				time = timeline.timeline().get_friendly_time(milliseconds)
	
				hours = time[2]
				mins = time[3]
				secs = time[4]
				milli = time[5]
	
				time_str =  "%02d:%02d:%02d" % (time[2], time[3], time[4])
	
				# get the thumbnail (or load default)
				pbThumb = item.get_thumbnail(51, 38)
				
				#find parent (if any)
				parent_name = item.parent
				if parent_name == None:
					match_iter = None
				else:
					match_iter = self.search_tree(self.MyTreestore, self.MyTreestore.iter_children(None), self.search_match, (1, "<b>%s</b>" % parent_name))
				# Add the file to the treeview
				self.MyTreestore.append(match_iter, [pbThumb,item.name, time_str, item.label])
			   
			elif isinstance(item, files.OpenShotFolder):
				#add folders
				pbThumb = gtk.gdk.pixbuf_new_from_file("%s/images/folder.png" % path)
				self.MyTreestore.append(None, [pbThumb, "<b>%s</b>" % item.name, None, None])
		
		# Check for NO files
		if self.project.project_folder.items.__len__() == 0:
			# Add the NO FILES message to the tree
			self.MyTreestore.append(None, [None, _("Choose a Video or Audio File to Begin"), "", ""])	   

			
	def search_tree(self,model, iter, func, data):
		while iter:
			if func(model, iter, data):
				return iter
			result = self.search_tree(model, model.iter_children(iter), func, data)
			if result: return result
			iter = model.iter_next(iter)
			
			
	def search_match(self,model, iter, data):
		column, key = data # data is a tuple containing column number, key
		value = model.get_value(iter, column)
		return value == key
	

	def refresh_thumb_view(self):
		"""Called when the thumbnail view is active"""
		view = self.icvFileIcons
		store = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
		for item in self.project.project_folder.items:
			#don't show folders in this view
			if isinstance(item, files.OpenShotFile):
				# get resized thumbnail image from the file object
				pbThumb = item.get_thumbnail(102, 76)

				# add to tree data
				store.append([pbThumb,item.name, item.label])
	
			view.set_item_width(130)
			view.set_model(store)
			view.set_text_column(1)
			view.set_pixbuf_column(0)
			view.set_text_column(2)
			
		 # Check for NO files
		if self.project.project_folder.items.__len__() == 0:
			#switch to the detail view
			mnu = mnuTree(None, None, form=self, project=self.project)
			mnu.on_mnuDetailView_activate(None)
			
	   
	def canvas_motion_notify(self, target, event):

		# update cursor variable
		if self.current_cursor[0] != None and self.current_cursor[3] == "canvas":
			# update cursor
			self.current_cursor = [None, int(event.x), int(event.y), ""]
		
			# reset the cursor icon
			self.MyCanvas.window.set_cursor(None)

		
		# update source of the cursor update
		self.current_cursor[3] = "canvas"
		
		
	#////////////////////
	def motion_cb(self, wid, context, x, y, time):
		# set the drag status
		context.drag_status(gtk.gdk.ACTION_COPY, time)
		
		# get the veritcal scrollbar value
		vertical_scroll_value = self.vscrollbar2.get_value()
		horizontal_scroll_value = self.hscrollbar2.get_value()

		# Add clip to canvas (upon the first event... but not subsequent events)
		if (self.item_detected == False):
			self.item_detected = True
			detail_view_visible = self.scrolledwindow1.get_property('visible')
			if detail_view_visible:
				# get the file info from the tree
				selection = self.myTree.get_selection()
				rows, selected = selection.get_selected_rows()
				iters = [rows.get_iter(path) for path in selected]
				for iter in iters:
					file_name = self.myTree.get_model().get_value(iter, 1)
					file_object = self.project.project_folder.FindFile(file_name)
					if file_object:
						file_length = file_object.length_in_seconds
					else:
						return
				
			else:
				#get the file info from the iconview
				selected = self.icvFileIcons.get_selected_items()
				if len(selected) > 1:
					return
				i = selected[0][0]
				model = self.icvFileIcons.get_model()
				file_name = model[i][1]
				file_object = self.project.project_folder.FindFile(file_name)
				if file_object:
					file_length = file_object.length_in_seconds
				else:
					return
		   
			# get a new track object
			self.new_clip_object = self.project.sequences[0].tracks[0].AddClip(file_name, "Gold", 0, float(0.0), float(file_length), file_object)

			# get pixels per second
			pixels_per_second = self.new_clip_object.parent.parent.get_pixels_per_second()
			self.new_clip_object.position_on_track = x / pixels_per_second
			
			# Render clip to timeline (at the current drag X coordinate)
			self.new_clip = self.new_clip_object.RenderClip()

		try:
			# get the x and y coordinate of the clip boundry
			new_x = x - self.new_clip.get_bounds().x1 + horizontal_scroll_value
			new_y = y - self.new_clip.get_bounds().y1 + vertical_scroll_value
	
			# don't allow the clip to slide past the beginning of the canvas
			total_x_diff = new_x - 40
			if (self.new_clip.get_bounds().x1 + total_x_diff < 0):
				total_x_diff = 0 - self.new_clip.get_bounds().x1
			
	
			# be sure that the clip is being dragged over a valid drop target (i.e. a track)
			if self.new_clip_object.is_valid_drop(self.new_clip.get_bounds().x1 + total_x_diff, self.new_clip.get_bounds().y1 + new_y - 25):
	
				# move the clip based on the event data
				self.new_clip.translate (total_x_diff, new_y - 25)
		
		except:
			# reset the drag n drop 
			self.item_detected = False
			self.new_clip_object = None



		return True


	def drop_cb(self, item, context, x, y, time):
		# complete the drag operation
		context.finish(True, False, time)

		# determine what cursor mode is enable (arrow, razor, snap, etc...)
		(isArrow, isRazor, isSnap, isResize) = self.get_toolbar_options()		
		
		if self.new_clip_object == None:
			return

		# get the track that the clip was dropped on		
		drop_track = self.new_clip_object.get_valid_drop(self.new_clip.get_bounds().x1, self.new_clip.get_bounds().y1)

		# update and reorder the clips on this track (in the object model)
		self.new_clip_object.update(self.new_clip.get_bounds().x1, self.new_clip.get_bounds().y1, drop_track)		 
		drop_track.reorder_clips()
		
		if drop_track:

			# deterime the direction of the drag
			if isSnap:
				distance_from_clip = self.new_clip_object.get_snap_difference(self.new_clip_object, self.new_clip)
			else:
				distance_from_clip = 0
			
			# move the clip object on the timeline to correct position (snapping to the y and x of the track)		
			self.new_clip.translate (distance_from_clip, drop_track.y_top - self.new_clip.get_bounds().y1 + 2)
			
			# update clip's settings
			self.new_clip_object.update(self.new_clip.get_bounds().x1, self.new_clip.get_bounds().y1, drop_track) 
			
			# move play head to top (over the clip)
			drop_track.parent.play_head.raise_(None)
			
			# Protect the visible button from track 0
			self.new_clip_object.protect_visible_button()

		else:
			# Remove clip, because of invalid parent
			self.new_clip_object.parent.clips.remove(self.new_clip_object)
			parent = self.new_clip.get_parent()
			child_num = parent.find_child (self.new_clip)
			parent.remove_child (child_num)

		# reset the drag n drop 
		self.item_detected = False
		self.new_clip_object = None

		return False
	#////////////////////




	def on_frmMain_destroy(self, widget, *args):
		# kill the threads
		if self.MyVideo:
			self.MyVideo.amAlive = False
			
		if self.project.thumbnailer:
			self.project.thumbnailer.amAlive = False
		
		# Quit the main loop, and exit the program
		self.quit()


	def on_mnuNewProject_activate(self, widget, *args):
		print "on_mnuNewProject_activate called with self.%s" % widget.get_name()
		NewProject.frmNewProject(mode="new", project=self.project)



	def on_mnuOpenProject_activate(self, widget, *args):
		print "on_mnuOpenProject_activate called with self.%s" % widget.get_name()
		OpenProject.frmOpenProject(project=self.project)


	def on_tlbOpenProject_clicked(self, widget, *args):
		OpenProject.frmOpenProject(project=self.project)


	def on_mnuImportFiles_activate(self, widget, *args):
		print "on_mnuImportFiles_activate called with self.%s" % widget.get_name()
		
		# show import file dialog
		AddFiles.frmAddFiles(form=self, project=self.project)


	def on_mnuSaveProject_activate(self, widget, *args):
		print "on_mnuSaveProject_activate called with self.%s" % widget.get_name()
		
		# call the save button
		self.on_tlbSave_clicked(widget)


	def on_mnuSaveProjectAs_activate(self, widget, *args):
		print "on_mnuSaveProjectAs_activate called with self.%s" % widget.get_name()
		NewProject.frmNewProject(mode="saveas", project=self.project)


	def on_mnuMakeMovie1_activate(self, widget, *args):
		print "on_mnuMakeMovie1_activate called with self.%s" % widget.get_name()

		# call toolbar button
		self.on_tlbMakeMovie_clicked(widget)

	def on_mnuQuit1_activate(self, widget, *args):
		print "on_mnuQuit1_activate called with self.%s" % widget.get_name()

		# Quit the main loop, and exit the program
		self.quit()

	def on_mnuPreferences_activate(self, widget, *args):
		print "on_mnuPreferences_activate called with self.%s" % widget.get_name()

		# coming soon
		messagebox.show("Error!", "This feature is still in development.")
		
		
	def on_mnuNewTitle_activate(self, widget, *args):
		print "on_mnuNewTitle_activate called with self.%s" % widget.get_name()

		# coming soon
		messagebox.show("Error!", "This feature is still in development. TIP: You can use any PNG or SVG image as a title, and the alpha and transparency are fully supported.")
		
		
	def on_mnuNewSequence_activate(self, widget, *args):
		print "on_mnuNewSequence_activate called with self.%s" % widget.get_name()

		# coming soon
		messagebox.show("Error!", "This feature is still in development.")
		
	def on_mnuAbout_activate(self, widget, *args):
		print "on_mnuAbout_activate called with self.%s" % widget.get_name()

		# Open About Dialog
		About.frmAbout()


	def on_tlbImportFiles_clicked(self, widget, *args):
		print "on_tlbImportFiles_clicked called with self.%s" % widget.get_name()
		
		# show import file dialog
		AddFiles.frmAddFiles(form=self, project=self.project)


	def on_tlbSave_clicked(self, widget, *args):
		print "on_tlbSave_clicked called with self.%s" % widget.get_name()
		# get the path of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0]))
		
		project_name = self.project.name
		project_folder = self.project.folder

		# determine if this project has been saved before
		if (project_folder != path):

			# save file exists... so just save again
			self.project.Save("%s/%s.osp" % (project_folder, project_name))

		else:

			# save file doesn't exist, so open "save as" window
			NewProject.frmNewProject(mode="saveas", project=self.project)


	def on_tlbMakeMovie_clicked(self, widget, *args):
		print "on_tlbMakeMovie_clicked called with self.%s" % widget.get_name()
		
		# show render message
		messagebox.show("Rendering!", "Your project is being rendered to %s/%s.mpeg" % (self.project.folder, self.project.name))
		
		# generate a new Westley XML file
		self.project.GenerateXML("westley.xml")
		
		if self.MyVideo == None:
			# Force SDL to write on our drawing area
			os.putenv('SDL_WINDOWID', str(self.videoscreen.window.xid))
			
			# We need to flush the XLib event loop otherwise we can't
			# access the XWindow which set_mode() requires
			gtk.gdk.flush()
			
			# play the video in it's own thread
			self.MyVideo = video.player(self.project, self, "westley.xml", mode="render")
			self.MyVideo.set_project(self.project, self, "westley.xml", mode="render")
			self.MyVideo.start()

		else:
			# re-load the xml
			self.MyVideo.set_project(self.project, self, "westley.xml", mode="render")
			self.MyVideo.load_xml()


	def on_treeFiles_drag_begin(self, widget, *args):
		context = args[0]

		# Get the drag icon
		play_image = gtk.image_new_from_file("%s/windows/glade/icons/plus.png" % self.openshot_path)
		pixbuf = play_image.get_pixbuf()
		context.set_icon_pixbuf(pixbuf, 15, 10)   
		
	def on_canvas_drag_motion(self, wid, context, x, y, time):
		print " *** motion detected"


	def on_canvas_drag_drop(self, widget, *args):
		print " *** DROP detected"


	def on_hsVideoProgress_value_changed(self, widget, *args):
		#print "on_hsVideoProgress_value_changed called with self.%s" % widget.get_name()		
		pass
	
	def on_tlbSnapshot_clicked(self, widget, *args):
		print "on_tlbSnapshot_clicked"
		
		# coming soon
		messagebox.show("Error!", "This feature is still in development.")
		
	
	def on_tlbPreviousMarker_clicked(self, widget, *args):
		print "on_tlbPreviousMarker_clicked"
		
		# get the previous marker object (if any)
		playhead_position = self.project.sequences[0].play_head_position
		marker = self.project.sequences[0].get_marker("left", playhead_position)
		is_playing = False
		if self.MyVideo:
			is_playing = self.MyVideo.isPlaying
		
		if marker:
			# determine frame number
			frame = self.project.fps() * marker.position_on_track
			
			# seek to this time
			if self.MyVideo:
				self.MyVideo.play()
				self.MyVideo.p.seek(int(frame))

				# check isPlaying
				if is_playing == False:
					self.MyVideo.pause()
				
			# move play-head
			self.project.sequences[0].move_play_head(marker.position_on_track)
		
	def on_tlbSeekBackward_clicked(self, widget, *args):
		print "on_tlbSeekBackward_clicked"

		if self.MyVideo:
			# get the current speed
			current_speed = self.MyVideo.p.get_speed()
			
			# calcualte new speed
			if current_speed - 1 == 0:
				new_speed = -1
			else:
				new_speed = current_speed - 1
			
			# set the new speed
			self.MyVideo.p.set_speed(new_speed)
			
			# update the preview tab label
			if new_speed == 1:
				self.lblVideoPreview.set_text("Video Preview")
			else:
				self.lblVideoPreview.set_text("Video Preview (%sX)" % int(new_speed))
			
	
	def on_tlbSeekForward_clicked(self, widget, *args):
		print "on_tlbSeekForward_clicked"
		
		if self.MyVideo:
			# get the current speed
			current_speed = self.MyVideo.p.get_speed()
			
			# calcualte new speed
			if current_speed + 1 == 0:
				new_speed = 1
			else:
				new_speed = current_speed + 1
			
			# set the new speed
			self.MyVideo.p.set_speed(new_speed)
			
			# update the preview tab label
			if new_speed == 1:
				self.lblVideoPreview.set_text("Video Preview")
			else:
				self.lblVideoPreview.set_text("Video Preview (%sX)" % int(new_speed))
			
	
	def on_tlbNextMarker_clicked(self, widget, *args):
		print "on_tlbNextMarker_clicked"
		
		# get the previous marker object (if any)
		playhead_position = self.project.sequences[0].play_head_position
		marker = self.project.sequences[0].get_marker("right", playhead_position)
		is_playing = False
		if self.MyVideo:
			is_playing = self.MyVideo.isPlaying
		
		if marker:
			# determine frame number
			frame = self.project.fps() * marker.position_on_track
			
			# seek to this time
			if self.MyVideo:
				self.MyVideo.play()
				self.MyVideo.p.seek(int(frame))

				# check isPlaying
				if is_playing == False:
					self.MyVideo.pause()
				
			# move play-head
			self.project.sequences[0].move_play_head(marker.position_on_track)
			
	
	def on_tlbAddMarker_clicked(self, widget, *args):
		print "on_tlbAddMarker_clicked"
		
		# get the current play_head position
		playhead_position = self.project.sequences[0].play_head_position
		
		# add a marker
		m = self.project.sequences[0].AddMarker("marker name", playhead_position)
		
		# refresh the screen
		if m:
			m.Render()
			
			# raise-play head
			#self.project.sequences[0].play_head.raise_(None)
			self.project.sequences[0].move_play_head(playhead_position)
		
		
		
	def on_tlbPrevious_clicked(self, widget, *args):
		print "on_tlbPrevious_clicked called with self.%s" % widget.get_name()

		if self.MyVideo:
			# seek to the first frame and reset the speed to 1X
			self.MyVideo.p.set_speed(1)
			self.MyVideo.p.seek(0)
			
			if self.MyVideo.isPlaying == False:
				self.MyVideo.p.set_speed(0)
			else:
				self.MyVideo.p.set_speed(1)
			
			# update video preview tab
			self.lblVideoPreview.set_text("Video Preview")
			

	def on_tlbPlay_clicked(self, widget, *args):
		print "on_tlbPlay_clicked called with self.%s" % widget.get_name()
		
		# determine location of OpenShot
		path = os.path.dirname(os.path.abspath(sys.argv[0]))
		
		# generate a new Westley XML file
		self.project.GenerateXML("westley.xml")
		
		if self.MyVideo == None:
			# Force SDL to write on our drawing area
			os.putenv('SDL_WINDOWID', str(self.videoscreen.window.xid))
			
			# We need to flush the XLib event loop otherwise we can't
			# access the XWindow which set_mode() requires
			gtk.gdk.flush()
			
			# play the video in it's own thread
			self.MyVideo = video.player(self.project, self, "westley.xml", mode="preview")
			self.MyVideo.start()


		else:
			# pause video 
			self.MyVideo.p.set_speed(0)

			# get current position
			position = self.MyVideo.p.position()
			is_playing = self.MyVideo.isPlaying	

			# ****************************
			# re-load the xml
			self.MyVideo.set_project(self.project, self, "westley.xml", mode="preview")
			self.MyVideo.load_xml()
			# ****************************

			# is video stopped?
			if is_playing == False:
				
				# start video
				self.MyVideo.play()
				
				# update video preview tab
				self.lblVideoPreview.set_text("Video Preview")

			else:
				
				# stop video
				self.MyVideo.pause()
				
				# update video preview tab
				self.lblVideoPreview.set_text("Video Preview (Paused)")


			# seek to the current position
			self.MyVideo.p.seek(position)
			


	def on_tlbNext_clicked(self, widget, *args):
		print "on_tlbNext_clicked called with self.%s" % widget.get_name()
		
		if self.MyVideo:
			# seek to the last frame and reset the speed to 1X
			self.MyVideo.p.set_speed(1)
			self.MyVideo.p.seek(self.MyVideo.p.get_length())
			
			if self.MyVideo.isPlaying == False:
				self.MyVideo.p.set_speed(0)
			else:
				self.MyVideo.p.set_speed(1)
			
			# update video preview tab
			self.lblVideoPreview.set_text("Video Preview")
			
			
	def on_tlbStop_clicked(self, widget, *args):
		print "on_tlbStop_clicked called with self.%s" % widget.get_name()
		
		if self.MyVideo:
			# stop thread from running
			self.MyVideo.pause()
			self.MyVideo.seek(1)
		

	def get_toolbar_options(self):
		""" return the options selected on the toolbar """
		
		isArrow = self.tlbArrow.get_active()
		isRazor = self.tlbRazor.get_active()
		isSnap = self.tlbSnap.get_active()
		isResize = self.tlbResize.get_active()
		
		# default to arrow mode
		if isRazor == False and isResize == False:
			isArrow = True
		
		return (isArrow, isRazor, isSnap, isResize)
	

	def on_tlbAddTrack_clicked(self, widget, *args):

		# Add a new track to the timeline		
		self.project.sequences[0].AddTrack("Track %s" % str(len(self.project.sequences[0].tracks) + 1))
		self.project.Render()


	def on_tlbRemoveTrack_clicked(self, widget, *args):
		print "on_tlbRemoveTrack_clicked called with self.%s" % widget.get_name()


	def on_tlbRazor_toggled(self, widget, *args):
		print "on_tlbRazor_clicked called with self.%s" % widget.get_name()

		# determine what cursor mode is enable (arrow, razor, snap, etc...)
		(isArrow, isRazor, isSnap, isResize) = self.get_toolbar_options()
		
		# only execute code if button is toggled (i.e. this gets called on un-toggle event also)
		if self.tlbRazor.get_active():
			
			# un-toggle the other toggles
			self.tlbArrow.set_active(False)
			self.tlbResize.set_active(False)
			
			# get the razor line image
			imgRazorLine = gtk.image_new_from_file("%s/windows/glade/icons/razor_line_with_razor.png" % self.openshot_path)
			pixRazorLine = imgRazorLine.get_pixbuf()
			
			# set cursor to normal
			self.MyCanvas.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.display_get_default(), pixRazorLine, 0, 28))
			
			
		# Keep the arrow selected, if no other toggles are selected
		if isRazor == False and isResize == False:
			self.tlbArrow.set_active(True)
			
			

	def on_tlbArrow_toggled(self, widget, *args):
		print "on_tlbArrow_clicked called with self.%s" % widget.get_name()
		
		# determine what cursor mode is enable (arrow, razor, snap, etc...)
		(isArrow, isRazor, isSnap, isResize) = self.get_toolbar_options()
		
		# only execute code if button is toggled (i.e. this gets called on un-toggle event also)
		if self.tlbArrow.get_active():   
		
			# un-toggle the other toggles
			self.tlbResize.set_active(False)
			self.tlbRazor.set_active(False)		
			
			# set cursor to normal
			self.MyCanvas.window.set_cursor(None)
			
			# Get the root group of the canvas
			#root_right = self.MyCanvas.get_root_item ()
	
			# Add a translucent blue rectangle to the canvas
			#image1 = goocanvas.Rect (parent = root_right,
									  #x = 12,
									  #y = 11,
									  #width = 100,
									  #height = 165,
									  #fill_color_rgba = 1401402200,
									  #stroke_color_rgba = 1401402500)
									  

		# Keep the arrow selected, if no other toggles are selected
		if isRazor == False and isResize == False:
			self.tlbArrow.set_active(True)
			
			
	def on_btnZoomIn_clicked(self, widget, *args):
		print "on_btnZoomIn_clicked"
		
		# get the value of the zoom slider
		zoom_slider = self.hsZoom.get_value()
		
		# zoom slower if too close
		if zoom_slider > 10:
			
			# subtract 5 units
			self.hsZoom.set_value(zoom_slider - 5)
		
		elif zoom_slider <= 5:
			# Move the zoom slider to the left
			if zoom_slider - 1 > 0:
				# subtract 5 units
				self.hsZoom.set_value(zoom_slider - 1)
			else:
				# set to 0
				self.hsZoom.set_value(0)
		
		elif zoom_slider <= 10:

			# subtract 5 units
			self.hsZoom.set_value(zoom_slider - 2)

		

			
		
	def on_btnZoomOu_clicked(self, widget, *args):
		print "on_btnZoomOu_clicked"
		
		# get the value of the zoom slider
		zoom_slider = self.hsZoom.get_value()
		

			
			
			
		# zoom slower if too close
		if zoom_slider > 11:
			
			# Move the zoom slider to the left
			if zoom_slider + 5 < 115:
				# add 5 units
				self.hsZoom.set_value(zoom_slider + 5)
			else:
				# set to 0
				self.hsZoom.set_value(115)
		
		elif zoom_slider < 4:
			
				# add 1 units
				self.hsZoom.set_value(zoom_slider + 1)
		
		elif zoom_slider <= 11:

			# add 2 units
			self.hsZoom.set_value(zoom_slider + 2)
			
			
		
	def on_hsZoom_value_changed(self, widget, *args):

		# get the value of the zoom slider (this value represents the number of seconds 
		# between the tick marks on the timeline ruler
		new_zoom_value = widget.get_value()

		# set the scale
		self.project.sequences[0].scale = int(new_zoom_value)
		
		# update zoom label
		self.lblZoomDetail.set_text("%d seconds" % new_zoom_value)
		
		# re-render the timeline with the new scale
		self.project.Render()
		
		
	def on_tlbResize_toggled(self, widget, *args):
		print "on_tlbResize_toggled called with self.%s" % widget.get_name()

		# determine what cursor mode is enable (arrow, razor, snap, etc...)
		(isArrow, isRazor, isSnap, isResize) = self.get_toolbar_options()  
		
		# only execute code if button is toggled (i.e. this gets called on un-toggle event also)
		if self.tlbResize.get_active():	
			
			# un-toggle the other toggles
			self.tlbArrow.set_active(False)
			self.tlbRazor.set_active(False)
			
		# Keep the arrow selected, if no other toggles are selected
		if isRazor == False and isResize == False:
			self.tlbArrow.set_active(True)
			
			
		
	def on_tlbSnap_toggled(self, widget, *args):
		print "on_tlbSnap_toggled called with self.%s" % widget.get_name()

		
		
		
	def on_drag_motion(self, wid, context, x, y, time):

		# Get the drag icon
		play_image = gtk.image_new_from_file("windows/glade/icons/png/AboutLogo.png")
		pixbuf = play_image.get_pixbuf()

		# Set the drag icon
		context.set_icon_pixbuf(pixbuf, 0, 0)


	def on_drag_data_received(self, widget, context, x, y, selection, target_type, timestamp):

		# get the list of files that were dropped in the tree
		uri = selection.data.strip()
		uri_splitted = uri.split() # we may have more than one file dropped
		
		if uri != self.last_files_added:	
			
			for uri in uri_splitted:
				# change cursor to "please wait"
				self.myTree.window.set_cursor(gtk.gdk.Cursor(150))
				
				# get the file path
				path = self.project.project_folder.get_file_path_from_dnd_dropped_uri(uri)
				
				# add file to current project
				self.project.project_folder.AddFile(path)

			# set these files as the 'last dragged'
			self.last_files_added = uri
			
		# refresh the form (i.e. add new items to the treeview)
		self.refresh()
		
		# set cursor to normal
		self.myTree.window.set_cursor(None)



	def on_scrolledwindow_Left_scroll_event(self, widget, *args):
		# Don't bubble up the scroll event.  This prevents the scroll wheel from 
		# scrolling the individual canvas.
		return True


	def on_scrolledwindow_Right_scroll_event(self, widget, *args):

		## Manually scroll the scrollbars
		if args[0].direction == gtk.gdk.SCROLL_DOWN:
			widget = self.vscrollbar2  
			vertical_value = widget.get_value() + 10

			# Update vertical scrollbar value
			widget.set_value(vertical_value)

			# Get horizontal value
			horizontal_scrollbar = self.hscrollbar2
			horizontal_value = horizontal_scrollbar.get_value()

			# scroll the canvases
			self.MyCanvas.scroll_to(horizontal_value, vertical_value)
			self.MyCanvas_Left.scroll_to(horizontal_value, vertical_value)
		else:
			widget = self.vscrollbar2	   
			vertical_value = widget.get_value() - 10

			# Update vertical scrollbar value
			widget.set_value(vertical_value)

			# Get horizontal value
			horizontal_scrollbar = self.hscrollbar2
			horizontal_value = horizontal_scrollbar.get_value()

			# scroll the canvases
			self.MyCanvas.scroll_to(horizontal_value, vertical_value)
			self.MyCanvas_Left.scroll_to(horizontal_value, vertical_value)

		# Don't bubble up the scroll event.  This prevents the scroll wheel from 
		# scrolling the individual canvas.   
		return True


	def on_vscrollbar2_value_changed(self, widget, *args):

		isinstance(widget, gtk.VScrollbar)		
		vertical_value = widget.get_value()

		# Get horizontal value
		horizontal_scrollbar = self.hscrollbar2
		horizontal_value = horizontal_scrollbar.get_value()

		# scroll the canvases
		self.MyCanvas.scroll_to(horizontal_value, vertical_value)
		self.MyCanvas_Left.scroll_to(horizontal_value, vertical_value)

	def on_treeFiles_button_press_event(self,treeview, event):
		"""This shows the right click menu"""
		if (event.button == 3):
			# determine which item the user right clicked on
			#path = treeview.get_path_at_pos(int(event.x),int(event.y))
			# Get the selection
			selection = treeview.get_selection()
			# Get the selected path(s)
			rows, selected = selection.get_selected_rows()
		   
			mnu = mnuTree(rows, selected, form=self, project=self.project)
			mnu.showmnu(event, treeview)
				
			return True
		
	def on_icvFileIcons_button_press_event(self, widget, event):
		"""This shows the right click menu"""
		if (event.button == 3):
			#show the right click menu
			mnu = mnuTree(None, None, form=self, project=self.project)
			mnu.showmnu(event, widget)


	def on_hscrollbar2_value_changed(self, widget, *args):

		isinstance(widget, gtk.HScrollbar)		
		horizontal_value = widget.get_value()

		# Get vertical value
		vertical_scrollbar = self.vscrollbar2
		vertical_value = vertical_scrollbar.get_value()

		# scroll the canvases
		self.MyCanvas.scroll_to(horizontal_value, vertical_value)

		
		
class mnuTrack(SimpleGladeApp):
	
	def __init__(self, rows, selected, path="Main.glade", root="mnuTrackPopup", domain="OpenShot", form=None, project=None, **kwargs):

		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator().lang.gettext
		
		self.form = form
		self.project = project
		
	def showmnu(self, event, widget):
		# show the popup menu
		self.mnuTrackPopup.show_all()
		self.mnuTrackPopup.popup( None, None, None, event.button, event.time)
		
		# selected track
		self.selected_track = widget


	def on_mnuAddTrackAbove_activate(self, event):
		print "on_mnuAddTrackAbove_activate clicked"
		
		# remove this track from it's parent sequence
		self.project.sequences[0].AddTrack("Track %s" % str(len(self.project.sequences[0].tracks) + 1), position="above", existing_track=self.selected_track)
		
		# refresh the interface
		self.project.Render()

		
	def on_mnuAddTrackBelow_activate(self, event):
		print "on_mnuAddTrackBelow_activate clicked"
		
		# remove this track from it's parent sequence
		self.project.sequences[0].AddTrack("Track %s" % str(len(self.project.sequences[0].tracks) + 1), position="below", existing_track=self.selected_track)
		
		# refresh the interface
		self.project.Render()
		
	def on_mnuRenameTrack_activate(self, event):
		print "on_mnuRenameTrack_activate clicked"
		
	def on_mnuRemoveTrack_activate(self, event):
		print "on_mnuRemoveTrack_activate clicked"
		
		# remove this track from it's parent sequence
		self.selected_track.parent.tracks.remove(self.selected_track)
		
		# refresh the interface
		self.project.Render()
		
	def on_mnuMoveTrackUp_activate(self, event):
		print "on_mnuMoveTrackUp_activate clicked"
		
		# get index of current track
		index_existing_track = self.selected_track.parent.tracks.index(self.selected_track)
		
		if index_existing_track > 0:
			# remove existing track
			self.selected_track.parent.tracks.remove(self.selected_track)
			
			# insert at new position
			self.selected_track.parent.tracks.insert(index_existing_track - 1, self.selected_track)
			
			# refresh the interface
			self.project.Render()
			
		
	def on_mnuMoveTrackDown_activate(self, event):
		print "on_mnuMoveTrackDown_activate clicked"
		
		
		# get index of current track
		index_existing_track = self.selected_track.parent.tracks.index(self.selected_track)
		
		if index_existing_track < len(self.selected_track.parent.tracks) - 1:
			# remove existing track
			self.selected_track.parent.tracks.remove(self.selected_track)
			
			# insert at new position
			self.selected_track.parent.tracks.insert(index_existing_track + 1, self.selected_track)
			
			# refresh the interface
			self.project.Render()
		
class mnuMarker(SimpleGladeApp):
	
	def __init__(self, rows, selected, path="Main.glade", root="mnuMarkerPopup", domain="OpenShot", form=None, project=None, **kwargs):

		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator().lang.gettext
		
		self.form = form
		self.project = project
		
		
	def showmnu(self, event, widget):
		# show the popup menu
		self.mnuMarkerPopup.show_all()
		self.mnuMarkerPopup.popup( None, None, None, event.button, event.time)
		
		# get selected widget
		self.selected_marker = widget
		
		
	def on_mnuRemoveMarker_activate(self, event):
		print "on_mnuRemoveClip_activate clicked"
		
		# remove this clip
		self.selected_marker.parent.markers.remove(self.selected_marker)
		#self.project.sequences[0].markers.remove(self.selected_marker)
		
		# refresh timeline
		self.project.Render()
		
		
		
class mnuClip(SimpleGladeApp):
	
	def __init__(self, rows, selected, path="Main.glade", root="mnuClipPopup", domain="OpenShot", form=None, project=None, **kwargs):

		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator().lang.gettext
		
		self.form = form
		self.project = project

		
	def showmnu(self, event, widget):
		# show the popup menu
		self.mnuClipPopup.show_all()
		self.mnuClipPopup.popup( None, None, None, event.button, event.time)
		
		# get selected widget
		self.selected_clip = widget


	def on_mnuRemoveClip_activate(self, event):
		print "on_mnuRemoveClip_activate clicked"
		
		# remove this clip
		self.selected_clip.parent.clips.remove(self.selected_clip)
		
		# refresh timeline
		self.project.Render()
		
		
	def on_mnuSliceandShuffle_activate(self, event):
		""" Cut a clip into many small pieces and shuffle them """
		print "on_mnuSliceandShuffle_activate"
		
		import random
		
		# init the list of clips
		min_random_length = 2
		max_random_length = 2
		list_of_clips = []
		end_of_clip = False
		
		clip_start = self.selected_clip.start_time
		clip_length = self.selected_clip.length()
		clip_max_length = self.selected_clip.max_length
		total_length = float(clip_start)
		position_on_track = float(self.selected_clip.position_on_track)
		
		
		# check for the minimun length validation
		if clip_length <= max_random_length:
			# show error
			messagebox.show("Error!", "You must select a clip which is more than %s seconds long." % max_random_length)
			return
			
		# loop through the clip, in increments of 2 to 5 seconds
		while not end_of_clip:
						
			# random number between 2 and 5
			random_length = float(random.Random().randint(min_random_length, max_random_length))

			# add clip properties to list
			list_of_clips.append([self.selected_clip.name, self.selected_clip.color, total_length, total_length + random_length, self.selected_clip.file_object, clip_max_length, random_length])

			# increment running total 
			total_length = total_length + random_length

			# kill loop when we reach the end
			if total_length + max_random_length >= clip_length:
				end_of_clip = True

				# add clip properties to list
				list_of_clips.append([self.selected_clip.name, self.selected_clip.color, total_length, total_length + (clip_length - total_length), self.selected_clip.file_object, clip_max_length, random_length])
			
			
		# shuffle the list of clips
		random.shuffle(list_of_clips)
		
		# actually add the clips to the track
		for clip in list_of_clips:
			# add clip
			Clip1 = self.selected_clip.parent.AddClip(clip[0], clip[1], position_on_track, clip[2], clip[3], clip[4])
			Clip1.max_length = clip[5]
			
			# increment position_on_track
			position_on_track = position_on_track + clip[6]
			
		
		# remove original clip from the timeline
		self.selected_clip.parent.clips.remove(self.selected_clip)

		# render timeline
		self.form.refresh()
		
		
	def on_mnuSliceandCut_activate(self, event):
		""" Cut a clip into many small pieces and shuffle them """
		print "on_mnuSliceandCut_activate"
		
		import random
		
		# init the list of clips
		min_random_length = 2
		max_random_length = 2
		list_of_clips = []
		end_of_clip = False
		
		clip_start = self.selected_clip.start_time
		clip_length = self.selected_clip.length()
		clip_max_length = self.selected_clip.max_length
		total_length = float(clip_start)
		position_on_track = float(self.selected_clip.position_on_track)
		should_add = True
		
		# check for the minimun length validation
		if clip_length <= max_random_length:
			# show error
			messagebox.show("Error!", "You must select a clip which is more than %s seconds long." % max_random_length)
			return
			
		# loop through the clip, in increments of 2 to 5 seconds
		while not end_of_clip:
						
			# random number between 2 and 5
			#random_length = float(random.Random().randint(min_random_length, max_random_length))

			# add clip properties to list
			if should_add:
				random_length = 2
				list_of_clips.append([self.selected_clip.name, self.selected_clip.color, total_length, total_length + random_length, self.selected_clip.file_object, clip_max_length, random_length])
				should_add = False
			else:
				random_length = 4
				should_add = True

			# increment running total 
			total_length = total_length + random_length

			# kill loop when we reach the end
			if total_length + max_random_length >= clip_length:
				end_of_clip = True

				# add clip properties to list
				if should_add:
					list_of_clips.append([self.selected_clip.name, self.selected_clip.color, total_length, total_length + (clip_length - total_length), self.selected_clip.file_object, clip_max_length, random_length])
					should_add = False
				else:
					should_add = True
			

		# actually add the clips to the track
		for clip in list_of_clips:
			# add clip
			Clip1 = self.selected_clip.parent.AddClip(clip[0], clip[1], position_on_track, clip[2], clip[3], clip[4])
			Clip1.max_length = clip[5]
			
			# increment position_on_track
			position_on_track = position_on_track + clip[6]
			
		
		# remove original clip from the timeline
		self.selected_clip.parent.clips.remove(self.selected_clip)

		# render timeline
		self.form.refresh()
		
		
class mnuTree(SimpleGladeApp):

	def __init__(self, rows, selected, path="Main.glade", root="mnuTreePopUp", domain="OpenShot", form=None, project=None, **kwargs):

		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator().lang.gettext

		self.mnuTree  = self.mnuTreePopUp
		self.model = rows
		self.selected = selected
		self.form = form
		self.project = project


	def showmnu(self, event, widget):
		#Show the right click menu.
		#dynamically show menu items depending on tree contents.
		#The Add Folder & Add File items are in the glade file - we always want them.
		name = widget.get_name()
		if self.project.project_folder.items.__len__() != 0:
			#Remove File
			mnuRemoveFile = gtk.ImageMenuItem(gtk.STOCK_DELETE)
			mnuRemoveFile.get_children()[0].set_label(_("Remove File(s)"))
			mnuRemoveFile.connect('activate',self.on_mnuRemoveFile_activate)
			self.mnuTree.add(mnuRemoveFile)
			
			

			#View options depend on what view is active
			if name == "treeFiles":
				mnuThumbView = gtk.ImageMenuItem(gtk.STOCK_MISSING_IMAGE)
				mnuThumbView.get_children()[0].set_label(_("Thumbnail view"))
				mnuThumbView.connect('activate',self.on_mnuThumbView_activate)
				self.mnuTree.add(mnuThumbView)
				
				#Move Files to folder
				mnuMoveFile = gtk.MenuItem(_("Move File(s) to Folder"))
				self.mnuTree.add(mnuMoveFile)
				
				folders =  self.project.project_folder.ListFolders()
				mnuSubMenu = gtk.Menu()
				#populate the sub menu with available folders
				for folder in folders:
					item = gtk.MenuItem(folder)
					item.connect("activate", self.move_file_to_folder, folder)
					mnuSubMenu.add(item)
				
				mnuMoveFile.set_submenu(mnuSubMenu)
			else:
				mnuDetailView = gtk.ImageMenuItem(gtk.STOCK_PROPERTIES)
				mnuDetailView.get_children()[0].set_label(_("Detail view"))
				mnuDetailView.connect('activate',self.on_mnuDetailView_activate)
				self.mnuTree.add(mnuDetailView)

			mnuSeparator = gtk.SeparatorMenuItem()
			self.mnuTree.add(mnuSeparator)

			#File properties
			mnuFileProperties = gtk.ImageMenuItem(gtk.STOCK_PROPERTIES)
			mnuFileProperties.get_children()[0].set_label(_("File Properties"))
			mnuFileProperties.connect('activate', self.on_FileProperties_activate)
			self.mnuTree.add(mnuFileProperties)

			self.mnuTree.show_all()

		self.mnuTree.popup( None, None, None, event.button, event.time)

		
	def move_file_to_folder(self, widget, folder):
		frm = self.form
		iters = [self.model.get_iter(path) for path in self.selected]
		for iter in iters:
			filename = self.model.get_value(iter, 1)		
			self.project.project_folder.AddParentToFile(filename, folder)
		frm.refresh() 
			
	def on_mnuAddFile_activate(self, event):
		AddFiles.frmAddFiles(form=self.form, project=self.project)

	def on_mnuAddNewFolder_activate(self, event):
		frm = frmFolders(form=self.form, project=self.project)
		frm.show()
		

	def on_mnuRemoveFile_activate(self, event):
		"""Removes a file from the treeview & project"""
		frm = self.form
		detail_view = frm.scrolledwindow1.get_property('visible')
		if detail_view == True:
			iters = [self.model.get_iter(path) for path in self.selected]
			for iter in iters:
				#remove from the file object
				filename = self.model.get_value(iter, 1)
				self.model.remove(iter)
				self.project.project_folder.RemoveFile(filename)
				
			frm.refresh()		
		else:
			#iconview is active
			selected = frm.icvFileIcons.get_selected_items()
			for item in selected:
				i = item[0]
				model = frm.icvFileIcons.get_model()
				filename = model[i][1]
				#remove the item from the project items list	
				self.project.project_folder.RemoveFile(filename)
				
			frm.refresh_thumb_view()
		
	   

	def on_FileProperties_activate(self, event):
		#show the file properties window
		frm = self.form
		
		detail_view = frm.scrolledwindow1.get_property('visible')
		if detail_view == True:
			if len(self.selected) > 1:
				#if more than 1 row selected, don't try and show the dialog
				return
			if len(self.selected) > 0:
				iters = [self.model.get_iter(path) for path in self.selected]
				for iter in iters:
					file = self.model.get_value(iter, 1)
			else: #nothing selected
				return
			
		else:
			#iconview is active
			selected = frm.icvFileIcons.get_selected_items()
			if len(selected) > 1:
				#more than 1 row selected, don't show the dialog
				return
			if len(selected) > 0:
				i = selected[0][0]
				model = frm.icvFileIcons.get_model()
				file = model[i][1]
			else: #nothing selected
				return
		#pass the file item to the file properties window
		file_item = self.project.project_folder.FindFile(file)
		if file_item:
			FileProperties.frmFileproperties(file_item, form=self, project=self.project)
			
		

	def on_mnuThumbView_activate(self, event):
		"""switch the view to the thumbnail view"""
		frm = self.form
		frm.scrolledwindow1.set_property('visible', False)
		child = frm.scrFileIcons
		frm.nbFiles.reorder_child(child, 0)
		frm.scrFileIcons.set_property('visible', True)
		frm.icvFileIcons.set_property('visible', True)
		frm.nbFiles.set_current_page(0)
		frm.refresh_thumb_view()

	def on_mnuDetailView_activate(self, event):
		"""switch the view to the treeview"""
		frm = self.form
		frm.scrFileIcons.set_property('visible', False)
		child = frm.scrolledwindow1
		frm.nbFiles.reorder_child(child, 0)
		frm.scrolledwindow1.set_property('visible', True)
		frm.myTree.set_property('visible', True)
		frm.nbFiles.set_current_page(0)
		frm.refresh()
	 
	#can't use this when treeview is in Multiple select mode	
	#def get_current_row(self):
	#	"""returns the current row from the treeview"""
	#	frm = current_main_form
	#	selection = frm.myTree.get_selection()
	#	model, iter = frm.myTree.get_selection().get_selected()
	#	return model, iter
		
class frmFolders(SimpleGladeApp):

	def __init__(self, path="Main.glade", root="frmFolder", domain="OpenShot", form=None, project=None, **kwargs):
		
		SimpleGladeApp.__init__(self, os.path.join("windows/glade", path), root, domain, **kwargs)
		
		# Add language support
		translator = Language_Init.Translator()
		_ = translator.lang.gettext

		self.FolderDlg  = self.frmFolder
		self.form = form
		self.project = project
		
	def show(self):
		self.FolderDlg.run()
		
	def on_btnCancel_clicked(self, widget, *args):
		self.FolderDlg.destroy()
		
	def on_btnOK_clicked(self, widget, *args):
		if self.txtFolderName.get_text() != "":
			folder_name = self.txtFolderName.get_text()
			self.project.project_folder.AddFolder(folder_name)		
		self.FolderDlg.destroy()




def main():
	frm_main = frmMain()
	frm_main.run()

if __name__ == "__main__":
	main()
